from pylem import MorphanHolder, MorphLanguage
import unittest

class TestBasicAnalyze(unittest.TestCase):

    def test_mama(self):
       holder = MorphanHolder(MorphLanguage.Russian)
       res = holder.is_in_dictionary("мама")
       self.assertTrue(res)

    def test_asokirko(self):
       holder = MorphanHolder(MorphLanguage.Russian)
       res = holder.is_in_dictionary("этогослованнет")
       self.assertFalse(res)

    def test_mother(self):
       holder = MorphanHolder(MorphLanguage.English)
       res = holder.is_in_dictionary("mother")
       self.assertTrue(res)

    def test_mutter(self):
       holder = MorphanHolder(MorphLanguage.German)
       res = holder.is_in_dictionary("mutter")
       self.assertTrue(res)



class TestMorphInfoJson(unittest.TestCase):
    def test_papa(self):
        holder = MorphanHolder(MorphLanguage.Russian)
        canon = [{'commonGrammems': 'anim', 'found': True, 'morphInfo': 'N mas,nom,sg', 'wordForm': 'ПАПА'}]
        res = holder.lemmatize_json("папа")
        self.assertEqual(len(canon), len(res))
        self.assertDictEqual(canon[0], res[0])


class TestSynthesis(unittest.TestCase):
    def test_stul(self):
        holder = MorphanHolder(MorphLanguage.Russian)
        answer = holder.synthesize("мама", "N fem,sg,gen")
        self.assertEqual(len(answer['forms']), 1)
        self.assertEqual("МАМЫ", answer['forms'][0])


if __name__ == '__main__':
    unittest.main()



def __bootstrap__():
    global __bootstrap__, __loader__, __file__
    import sys, pkg_resources, importlib.util, os
    library_name = "pylem_binary"
    def find_compiled_library():
        f  =  os.path.dirname(__file__)
        if  len(f) == 0:
            f =  "."
        for i in os.listdir(f):
            if i.startswith(library_name) and (i.endswith('.pyd') or i.endswith('.so')):
                return i
        raise Exception ("no binary library found to load")
    compiled_lib_path = find_compiled_library()
    print ("compiled_lib_path = {}".format(compiled_lib_path))
    print(sys.modules)
    __file__ = pkg_resources.resource_filename(library_name, compiled_lib_path)
    __loader__ = None; 
    del __bootstrap__, __loader__
    spec = importlib.util.spec_from_file_location(library_name, __file__)
    mod = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(mod)
__bootstrap__()




import os
import json
from .pylem_binary import *


class MorphLanguage:
    Russian = 1
    English = 2
    German = 3

    @staticmethod
    def check_language(langua):
        return langua == MorphLanguage.Russian or langua == MorphLanguage.English or langua == MorphLanguage.German

    @staticmethod
    def GetStrByLanguage(langua):
        if langua == MorphLanguage.Russian:
            return "Russian"
        elif langua == MorphLanguage.English:
            return "English"
        elif langua == MorphLanguage.German:
            return "German"
        else:
            return "unknown"


class LemmaInfo:
    def __init__(self, predicted, lemma = None, part_of_speech = None, morph_features=None, predicted_by=None):
        self.predicted = predicted
        self.lemma = lemma
        self.part_of_speech = part_of_speech
        self.morph_features = morph_features
        self.predicted_by = predicted_by


class MorphanHolder:

    def __init__(self, langua):
        assert MorphLanguage.check_language(langua)
        self.language = langua
        path = os.path.join(os.path.dirname(__file__), 'Dicts/Morph', MorphLanguage.GetStrByLanguage(langua))
        ret = load_morphology(int(langua), path)

    def lemmatize_json(self, word, all_forms=False):
        r = lemmatize_json(self.language, word, all_forms)
        return json.loads(r)

    def lemmatize(self, word):
        lemm_infos_in_json = self.lemmatize_json(word, False)
        for l in lemm_infos_in_json:
            #{'commonGrammems': 'inanim', 'found': False, 'morphInfo': 'N neu,fem,mas,prp,ins,acc,dat,gen,nom,sg,pl', 'srcNorm': 'НЕУБИВАЙМЕНЯ', 'wordForm': 'ААБ'}
            grammems =  set(l.get('commonGrammems', '').split(','))
            i = l['morphInfo'].find(' ')
            if i == -1:
                pos = l['morphInfo']
            else:
                pos = l['morphInfo'][0:i]
                grammems.update((l['morphInfo'][i+1:].split(',')))
            lemm_info = LemmaInfo(predicted=not l['found'],
                                  lemma=l['wordForm'],
                                  part_of_speech=pos,
                                  morph_features=grammems,
                                  predicted_by=l.get('srcNorm'))
            yield lemm_info

    def is_in_dictionary(self, word):
        r = is_in_dictionary(self.language, word)
        return r

    def synthesize(self, word, all_forms=False):
        r = synthesize(self.language, word, all_forms)
        return json.loads(r)


class MorphSourceDictHolder:
    def __init__(self, mwz_path):
        self.mwz_path = mwz_path
        if not os.path.exists(mwz_path):
            raise OSError("cannot find mwz project {}".format(mwz_path))
        load_mwz_project(self.mwz_path)

    def predict_lemm(self, word, suf_len: int, minimal_frequence: int):
        predicted = predict_lemm(self.mwz_path, word, suf_len, minimal_frequence)
        return predicted



from pylem import MorphSourceDictHolder
import unittest
import os


class TestLemPredict(unittest.TestCase):

    def test_surname_predict(self):
        mwz_path = os.path.join(os.path.dirname(__file__), '../morph_dict/data/Russian/project.mwz')
        holder = MorphSourceDictHolder(mwz_path)
        r = holder.predict_lemm("асокирко", 3, 2)
        self.assertGreater(len(r), 0)

if __name__ == '__main__':
    unittest.main()
